function [f,varargout] = Fbox(A,a,b,theta,x0,y0,N,varargin)
% FBOX 2-D Fourier data of a rectangle image (table lookup).
%    [F] = FBOX(A, a, b, THETA, X0, Y0, N) returns a 2-D k-space data of a
%    rectangle with width a and hieght b and intensity A. THETA, X0, and Y0
%    are used for the k-space data with 2-D rigid body transformation.
%    THETA is determined in degree unit with a counterclockwise direction. 
%    X0 and Y0 are a translation along X and Y directions respectively from
%    the original image. N is used to determine the size of an image or
%    datapoints along X and Y directions. The k-space data is given in a
%    square matrix of size N(1)xN(2), or NxN if N is not a 1x2 matrix. The
%    default Field of View along X and Y directions are 128 for both
%    directions. The scaling factors in X and Y directions are 1. The
%    k-space data is not corrupted by complex Gaussian noise.
%    
%    [F] = FBOX(A, a, b, THETA, X0, Y0, N, FOVX, FOVY) returns a 2-D 
%    k-space data of a rectangle with specified parameters mentioned above. 
%    FOVX and FOVY are used to determine the Field of View (FOV) along X 
%    and Y directions respectively. The default scaling factors in X and Y
%    directions are used.
%    
%    [F] = FBOX(A, a, b, THETA, X0, Y0, N, FOVX, FOVY, SNR) returns a 2-D
%    k-space data of a rectangle with specified parameters mentioned above.
%    Complex Gaussian noise with a given SNR (in dB) is added into k-space
%    domain data. The default scaling factors in X and Y directions are used.
%    
%    [F] = FBOX(A, a, b, THETA, X0, Y0, N, FOVX, FOVY, SCALEX, SCALEY)
%    returns a 2-D k-space data of a rectangle with specified parameters.
%    SCALEX and SCALEY are used to determine the scaling factors in X and Y
%    directions respectively.
%    
%    [F] = FBOX(A, a, b, THETA, X0, Y0, N, FOVX, FOVY, SNR, SCALEX, SCALEY)
%    returns a 2-D k-space data of a rectangle with specified parameters.
%    Complex Gaussian noise with a given SNR (in dB) is added into k-space
%    domain data. SCALEX and SCALEY are used to determine the scaling
%    factors in X and Y directions respectively.
%    
%    [F, I] = FBOX(A, a, b, THETA, X0, Y0, N,...) also returns the image
%    resulted from the inverse Fourier transform of the generated k-space
%    data.
%
%    [F, I, KX, KY] = FBOX(A, a, b, THETA, X0, Y0, N,...) also returns the
%    image resulted from the inverse Fourier transform of the generated
%    k-space data and the k-space coordinates (KX, KY) corresponding to the
%    generated k-space data of a rectangle with specified input parameters.
% 
%    See also FELLIPSE, FBOXR, FBOXS, and DCEK.
%    Created and modified by Yuttapong Jiraraksopakun
%
%
%    Copyrigt 2007, Jim Ji, Texas A&M University. All Rights Reserved. 
%

error(nargchk(7, 12, nargin))

error(nargoutchk(1, 4, nargout));

switch length(N)
    case 1
        Nx = N;
        Ny = N;
    case 2
        Nx = N(1);
        Ny = N(2);
    otherwise
        error('Invalid dimension: N')
end

switch nargin
    case 7
        FOVx = 128;
        FOVy = 128;
        SNR = [];
        scalex = 1;
        scaley = 1;
    case 9
        FOVx = varargin{1};
        FOVy = varargin{2};
        SNR = [];
        scalex = 1;
        scaley = 1;
    case 10
        FOVx = varargin{1};
        FOVy = varargin{2};
        SNR = varargin{3};
        scalex = 1;
        scaley = 1;
    case 11
        FOVx = varargin{1};
        FOVy = varargin{2};
        SNR = [];
        scalex = varargin{3};
        scaley = varargin{4};
    case 12
        FOVx = varargin{1};
        FOVy = varargin{2};
        SNR = varargin{3};
        scalex = varargin{4};
        scaley = varargin{5};
    otherwise
        error('Ambiguous input arguments')
end

switch nargout
    case 3
        error( 'Too few output arguments' )
    otherwise
end

C = (Nx/FOVx)*(Ny/FOVy)*A*(4*a*b)/abs(scalex*scaley);
[fx,fy] = meshgrid(-Nx/2:Nx/2-1,Ny/2-1:-1:-Ny/2);
kx = 1/(FOVx)*fx;
ky = 1/(FOVy)*fy;
rad = theta*pi/180;
kxr = 1/scalex*(kx*cos(rad)+ky*sin(rad));
kyr = 1/scaley*(-kx*sin(rad)+ky*cos(rad));
f = C*sinc(2*b*kyr).*sinc(2*a*kxr).*exp(-i*2*pi*(kx*x0+ky*y0));

if ~isempty(SNR)
    sigmakx = sqrt((Nx*Ny)*(A^2)*power(10,-SNR/10)/(2));
    N = random('Normal',0,sigmakx,prod(size(f)),1)+sqrt(-1)*random('Normal',0,sigmakx,prod(size(f)),1);
    N = reshape(N,size(f));
    f = f+N;
end

if nargout > 1
    if nargout == 2
        varargout{1} = abs(ifftshift(ifft2(ifftshift(f))));
    else
        varargout{1} = abs(ifftshift(ifft2(ifftshift(f))));
        varargout{2} = kx;
        varargout{3} = ky;
    end
end