function [f,varargout] = Fboxr(A,a,b,theta,x0,y0,N,varargin)
% FBOXR 2-D Fourier data of a rectangle image with radial trajectory (table lookup).
%    [F] = FBOXR(A, a, b, THETA, X0, Y0, N) returns a 2-D k-space data of a
%    rectangle with width a and hieght b and intensity A. THETA, X0, and Y0
%    are used for the k-space data with 2-D rigid body transformation.
%    THETA is determined in degree unit with a counterclockwise direction. 
%    X0 and Y0 are a translation along X and Y directions respectively from
%    the original image. N is used to determine the size of an image or
%    datapoints along X and Y directions. The k-space data is given in a
%    square matrix of size N(1)xN(2), or NxN if N is not a 1x2 matrix. The
%    default Field of View along X and Y directions are 128 for both
%    directions. The k-space data is not corrupted by complex Gaussian
%    noise, and the default scaling factors in X and Y directions are 1.
%    
%    [F] = FBOXR(A, a, b, THETA, X0, Y0, N, FOVX, FOVY) returns a 2-D 
%    k-space data of a rectangle with specified parameters mentioned above. 
%    FOVX and FOVY are used to determine the Field of View (FOV) along X 
%    and Y directions respectively. The default scaling factors in X and Y
%    directions are used.
%    
%    [F] = FBOXR(A, a, b, THETA, X0, Y0, N, FOVX, FOVY, SNR) returns
%    a 2-D k-space data of a rectangle with specified parameters mentioned
%    above. Complex Gaussian noise with a given SNR (in dB) is added into
%    k-space domain for generating k-space data corrupted by noise. The
%    default scaling factors in X and Y directions are used.
%    
%    [F] = FBOXR(A, a, b, THETA, X0, Y0, N, FOVX, FOVY, SNR, SCALEX, SCALEY)
%    returns a 2-D k-space data of a rectangle with specified parameters.
%    SCALEX and SCALEY are used to determine the scaling factors in X and Y
%    directions respectively.
%    
%    [F, I] = FBOXR(A, a, b, THETA, X0, Y0, N, INT_METHOD, FILTER) also
%    returns an image reconstructed from the inverse Radon transform of the
%    generated k-space data with specified parameteres mention above.
%    INT_METHOD and FILTER are parameters for inverse Radon transform. If
%    not specified, FBOXR uses bilinear interpolation and Ram-Lak filter as
%    the default values. No noise corrupts the reconstructed image, and the
%    default FOVs and scaling factors in X and Y directions are used.
%
%    [F, I] = FBOXR(A, a, b, THETA, X0, Y0, N, INT_METHOD, FILTER, SNR)
%    also returns an image reconstructed from the inverse Radon transform
%    of the generated k-space data with specified parameteres mention
%    above. Complex Gaussian noise with a given SNR (in dB) is added into
%    k-space domain before image reconstruction, and the default FOVs and
%    scaling factors in X and Y directions are used.
%    
%    [F, I] = FBOXR(A, a, b, THETA, X0, Y0, N, FOVX, FOVY, INT_METHOD, FILTER)
%    also returns an image reconstructed from the inverse Radon transform
%    of the generated k-space data with specified parameteres mentioned
%    above. FOVx and FOVy are used to determine the Field of View (FOV)
%    along X and Y directions respectively. No noise corrupts the
%    reconstructed image, and the default scaling factors in X and Y
%    directions are used.
%
%    [F, I] = FBOXR(A, a, b, THETA, X0, Y0, N, FOVX, FOVY, INT_METHOD, FILTER, SNR)
%    also returns an image reconstructed from the inverse Radon transform
%    of the generated k-space data with specified parameteres mentioned
%    above. Complex Gaussian noise with a given SNR (in dB) is added into
%    k-space domain before image reconstruction, and the default scaling
%    factors in X and Y directions are used.
%
%    [F, I] = FBOXR(A, a, b, THETA, X0, Y0, N, FOVX, FOVY, INT_METHOD, FILTER, SNR, SCALEX, SCALEY)
%    also returns an image reconstructed from the inverse Radon transform
%    of the generated k-space data with specified parameteres mentioned
%    above. SCALEX and SCALEY are used to determine the scaling factors in
%    X and Y directions respectively.
%
%    [F, KX, KY] = FBOXR(A, a, b, THETA, X0, Y0, N,...) also returns the
%    k-space coordinates (KX, KY) corresponding to the generated k-space
%    data of a rectangle with specified input parameters.
% 
%    [F, I, KX, KY] = FBOXR(A, a, b, THETA, X0, Y0, N,...) also returns an
%    image resulted from the inverse Radon transform of the generated
%    k-space data and the k-space coordinates (KX, KY) corresponding to the
%    generated k-space data of a rectangle with specified input parameters.
% 
%    See also IRADON, FBOX, FBOXS, FELLIPSE, FELLIPSER, DCEKR, and DCEK.
%
%    Created and modified by Yuttapong Jiraraksopakun
%
%
%    Copyrigt 2007, Jim Ji, Texas A&M University. All Rights Reserved. 
%

error(nargchk(7, 14, nargin))

error(nargoutchk(1, 4, nargout));

switch length(N)
    case 1
        Nx = N;
        Ny = N;
    case 2
        Nx = N(1);
        Ny = N(2);
    otherwise
        error('Invalid dimension: N')
end

switch nargout
    case {1,3}
        switch nargin
            case 7
                FOVx = 128;
                FOVy = 128;
                interp_method = 'linear';
                filter = 'Ram-Lak';
                SNR = [];
                scalex = 1;
                scaley = 1;
            case 9
                FOVx = varargin{1};
                FOVy = varargin{2};
                interp_method = 'linear';
                filter = 'Ram-Lak';
                SNR = [];
                scalex = 1;
                scaley = 1;
            case 10
                FOVx = varargin{1};
                FOVy = varargin{2};
                interp_method = 'linear';
                filter = 'Ram-Lak';
                SNR = varargin{3};
                scalex = 1;
                scaley = 1;
            case 12
                FOVx = varargin{1};
                FOVy = varargin{2};
                interp_method = 'linear';
                filter = 'Ram-Lak';
                SNR = varargin{3};
                scalex = varargin{4};
                scaley = varargin{5};
            otherwise
                error('Ambiguous input arguments')
        end
    otherwise % case {2,4}
        switch nargin
            case 7
                FOVx = 128;
                FOVy = 128;
                interp_method = 'linear';
                filter = 'Ram-Lak';
                SNR = [];
                scalex = 1;
                scaley = 1;
            case 9
                FOVx = 128;
                FOVy = 128;
                interp_method = varargin{1};
                filter = varargin{2};
                SNR = [];
                scalex = 1;
                scaley = 1;
            case 10
                FOVx = 128;
                FOVy = 128;
                interp_method = varargin{1};
                filter = varargin{2};
                SNR = varargin{3};
                scalex = 1;
                scaley = 1;
            case 11
                FOVx = varargin{1};
                FOVy = varargin{2};
                interp_method = varargin{3};
                filter = varargin{4};
                SNR = [];
                scalex = 1;
                scaley = 1;
            case 12
                FOVx = varargin{1};
                FOVy = varargin{2};
                interp_method = varargin{3};
                filter = varargin{4};
                SNR = varargin{5};
                scalex = 1;
                scaley = 1;
            case 14
                FOVx = varargin{1};
                FOVy = varargin{2};
                interp_method = varargin{3};
                filter = varargin{4};
                SNR = varargin{5};
                scalex = varargin{6};
                scaley = varargin{7};
            otherwise
                error('Ambiguous input arguments')
        end
end

C = (Nx/FOVx)*(Ny/FOVy)*A*(4*a*b)/abs(scalex*scaley);
Nmax = max(N);
[rho, phi] = meshgrid(-Nmax/2:Nmax/2-1,0:pi/Nmax:pi-pi/Nmax);
kx = 1/(FOVx)*rho.*cos(phi);
ky = 1/(FOVy)*rho.*sin(phi);

rad = theta*pi/180;
kxr = 1/scalex*(kx*cos(rad)+ky*sin(rad));
kyr = 1/scaley*(-kx*sin(rad)+ky*cos(rad));
f = C*sinc(2*b*kyr).*sinc(2*a*kxr).*exp(-i*2*pi*(kx*x0+ky*y0));

if ~isempty(SNR)
    sigmakx = sqrt((Nx*Ny)*(A^2)*power(10,-SNR/10)/(2));
    N = random('Normal',0,sigmakx,prod(size(f)),1)+sqrt(-1)*random('Normal',0,sigmakx,prod(size(f)),1);
    N = reshape(N,size(f));
    f = f+N;
end

if nargout > 1
    switch nargout
        case 2
%             varargout{1} = abs(Recon_arbitrarykspace(kx,ky,f,Nx,Ny,'linear',FOVx,FOVy));
            varargout{1} = abs(fbprec( f, [0:Nmax-1]*180/Nmax, Nmax, interp_method, filter ));
        case 3
            varargout{1} = kx;
            varargout{2} = ky;
        otherwise
%             varargout{1} = abs(Recon_arbitrarykspace(kx,ky,f,Nx,Ny,'linear',FOVx,FOVy));
            varargout{1} = abs(fbprec( f, [0:Nmax-1]*180/Nmax, Nmax, interp_method, filter ));
            varargout{2} = kx;
            varargout{3} = ky;
    end
end