function [f,varargout] = Fcirc(A,r,theta,x0,y0,N,varargin)
% FCIRC 2-D Fourier data of a circle image (table lookup).
%    [F] = FCIRC(A, R, THETA, X0, Y0, N) returns a 2-D k-space data of a
%    circle with radius R and intensity A. THETA, X0, and Y0 are used for
%    the k-space data with 2-D rigid body transformation. THETA is
%    determined in degree unit with a counterclockwise direction. X0 and Y0
%    are a translation along X and Y directions respectively from the
%    original image. N is used to determine the size of an image or
%    datapoints along X and Y directions. The k-space data is given in a
%    square matrix of size N(1)xN(2), or NxN if N is not a 1x2 matrix. The
%    default Field of View along X and Y directions are 128 for both
%    directions. The scaling factors in X and Y directions are 1.
%    
%    [F] = FCIRC(A, R, THETA, X0, Y0, N, FOVX, FOVY) returns a 2-D k-space
%    data of a circle with specified parameters mentioned above. FOVX and
%    FOVY are used to determine the Field of View (FOV) along X and Y
%    directions respectively. The default scaling factors in X and Y
%    directions are used.
%    
%    [F] = FCIRC(A, R, THETA, X0, Y0, N, FOVX, FOVY, SCALEX, SCALEY)
%    returns a 2-D k-space data of a circle with specified parameters.
%    SCALEX and SCALEY are used to determine the scaling factors in X and Y
%    directions respectively.
%    
%    [F, I] = FCIRC(A, R, THETA, X0, Y0, N,...) also returns the image
%    resulted from the inverse Fourier transform of the generated k-space
%    data.
%    
%    [F, I, KX, KY] = FCIRC(A, R, THETA, X0, Y0, N,...) also returns the
%    image resulted from the inverse Fourier transform of the generated
%    k-space data and the k-space coordinates (KX, KY) corresponding to the
%    generated k-space data of a circle with specified input parameters.
% 
%    See also FBOX, FCUBE, FSPHERE, and DCEK.
%
%    Copyrigt 2007, Jim Ji, Texas A&M University. All Rights Reserved. 
%


error(nargchk(6, 10, nargin))

error(nargoutchk(1, 4, nargout));

switch length(N)
    case 1
        Nx = N;
        Ny = N;
    case 2
        Nx = N(1);
        Ny = N(2);
    otherwise
        error('Invalid dimension: N')
end

switch nargin
    case 6
        FOVx = 128;
        FOVy = 128;
        scalex = 1;
        scaley = 1;
    case 8
        FOVx = varargin{1};
        FOVy = varargin{2};
        scalex = 1;
        scaley = 1;
    case 10
        FOVx = varargin{1};
        FOVy = varargin{2};
        scalex = varargin{3};
        scaley = varargin{4};
    otherwise
        error('Ambiguous input arguments')
end

switch nargout
    case 3
        error( 'Too few output arguments' )
    otherwise
end

C = (Nx/FOVx)*(Ny/FOVy)*A*(2*pi*r^2)/abs(scalex*scaley);
[fx,fy] = meshgrid(-Nx/2:Nx/2-1,Ny/2-1:-1:-Ny/2);
kx = 1/(FOVx)*fx;
ky = 1/(FOVy)*fy;
rad = theta*pi/180;
kxr = 1/scalex*(kx*cos(rad)+ky*sin(rad));
kyr = 1/scaley*(-kx*sin(rad)+ky*cos(rad));
rho = sqrt(kxr.^2+kyr.^2);
f = C*jinc(2*pi*r*rho).*exp(-i*2*pi*(kx*x0+ky*y0));

if nargout > 1
    if nargout == 2
        varargout{1} = abs(ifftshift(ifft2(ifftshift(f))));
    else
        varargout{1} = abs(ifftshift(ifft2(ifftshift(f))));
        varargout{2} = kx;
        varargout{3} = ky;
    end
end