function [f,varargout] = Fellipses(A,r,a,b,theta,x0,y0,N,varargin)
% FELLIPSE 2-D Fourier data of an ellipse image with spiral trajectory (table lookup).
%    [F] = FELLIPSES(A, R, a, b, THETA, X0, Y0, N) returns a 2-D k-space
%    data of an ellipse with intensity A. An ellipse will be constructed
%    from a circle with radius R and scaling factors a and b along X and Y
%    directions respectively. THETA, X0, and Y0 are used for the k-space
%    data with 2-D rigid body transformation. THETA is determined in degree
%    unit with a counterclockwise direction. X0 and Y0 are a translation
%    along X and Y directions respectively from the original image. N is
%    used to determine the size of an image or datapoints along X and Y
%    directions. The k-space data is given in a square matrix of size
%    N(1)xN(2), or NxN if N is not a 1x2 matrix. The default Field of View
%    along X and Y directions are 128 for both directions. The spiral
%    trajectory is generated according to the equation:
%
%         (kx(t),ky(t)) = A*t*(cos(2*pi*w*t+beta),sin(2*pi*w*t+beta))
%
%    where t ranges in the interval [0,1]; A is set to be 0.8*max(N)/2;
%    w is the frequency, 256 for the default value; and beta is set to be 0. 
%    The k-space data is not corrupted by complex Gaussian noise.
%    
%    [F] = FELLIPSES(A, R, a, b, THETA, X0, Y0, N, FOVX, FOVY) returns a 2-D
%    k-space data of an ellipse with specified parameters mentioned above.
%    FOVX and FOVY are determined for the Field of View (FOV) along X and Y
%    directions respectively. The default frequency w is used.
%      
%    [F] = FELLIPSES(A, R, a, b, THETA, X0, Y0, N, FOVX, FOVY, SNR) returns
%    a 2-D k-space data of an ellipse with specified parameters mentioned
%    above. Complex Gaussian noise with a given SNR (in dB) is added into
%    k-space domain for generating k-space data corrupted by noise. The
%    default frequency w is used.
%      
%    [F] = FELLIPSES(A, R, a, b, THETA, X0, Y0, N, FOVX, FOVY, SNR, W)
%    returns a 2-D k-space data of an ellipse with specified parameters
%    mentioned above. W is the frequency for generating a spiral trajectory.
%      
%    [F, I] = FELLIPSES(A, R, a, b, THETA, X0, Y0, N, INT_METHOD) also
%    returns the image resulted from the inverse Fourier transform of the
%    generated k-space data with parameters mentioned above. INT_METHOD is
%    determined for 2-D interpolation from spiral to rectilinear k-space
%    data. If not specified, FELLIPSES uses bilinear interpolation as a
%    default value. No noise corrupts the reconstructed image, and the
%    default FOVs and frequency w are used.
%    
%    [F, I] = FELLIPSES(A, R, a, b, THETA, X0, Y0, N, INT_METHOD, SNR) also
%    returns the image resulted from the inverse Fourier transform of the
%    generated k-space data with parameters mentioned above. Complex
%    Gaussian noise with a given SNR (in dB) is added into k-space domain
%    before image reconstruction. The default FOVs and frequency w are used.
%
%    [F, I] = FELLIPSES(A, R, a, b, THETA, X0, Y0, N, FOVX, FOVY, INT_METHOD) 
%    also returns the image resulted from the inverse Fourier transform of
%    the generated k-space data with parameters mentioned above. FOVx and
%    FOVy are used to determine the Field of View (FOV) along X and Y
%    directions respectively. No noise corrupts the reconstructed image,
%    and the default frequency w is used.
%    
%    [F, I] = FELLIPSES(A, R, a, b, THETA, X0, Y0, N, FOVX, FOVY, INT_METHOD, SNR) 
%    also returns the image resulted from the inverse Fourier transform of
%    the generated k-space data with parameters mentioned above. Complex
%    Gaussian noise with a given SNR (in dB) is added into k-space domain
%    before image reconstruction. The default frequency w is used.
%
%    [F, I] = FELLIPSES(A, R, a, b, THETA, X0, Y0, N, FOVX, FOVY, INT_METHOD, SNR, W)
%    also returns the image resulted from the inverse Fourier transform of
%    the generated k-space data with parameters mentioned above. W is the
%    frequency for generating a spiral trajectory.
%
%    [F, KX, KY] = FELLIPSES(A, R, a, b, THETA, X0, Y0, N,...) also returns
%    the k-space coordinates (KX, KY) corresponding to the generated
%    k-space data of an ellipse with specified input parameters.
% 
%    [F, I, KX, KY] = FELLIPSES(A, R, a, b, THETA, X0, Y0, N,...) also
%    returns the image resulted from the inverse Fourier transform of the
%    generated k-space data and the k-space coordinates (KX, KY)
%    corresponding to the generated k-space data of an ellipse with
%    specified input parameters.
% 
%    See also FBOX, FBOXS, FELLIPSE, FELLIPSER, DCEKS, and DCEK.
%    Created and modified by Yuttapong Jiraraksopakun
%
%
%    Copyrigt 2007, Jim Ji, Texas A&M University. All Rights Reserved. 
%

error(nargchk(8, 13, nargin))

error(nargoutchk(1, 4, nargout));

if (a==0) | (b==0)
    error('a or b must not be zero')
end

switch length(N)
    case 1
        Nx = N;
        Ny = N;
    case 2
        Nx = N(1);
        Ny = N(2);
    otherwise
        error('Invalid dimension: N')
end

switch nargout
    case {1,3}
        switch nargin
            case 8
                FOVx = 128;
                FOVy = 128;
                interp_method = 'linear';
                SNR = [];
                w = 160;
            case 10
                FOVx = varargin{1};
                FOVy = varargin{2};
                interp_method = 'linear';
                SNR = [];
                w = 160;
            case 11
                FOVx = varargin{1};
                FOVy = varargin{2};
                interp_method = 'linear';
                SNR = varargin{3};
                w = 160;
            case 12
                FOVx = varargin{1};
                FOVy = varargin{2};
                interp_method = 'linear';
                SNR = varargin{3};
                w = varargin{4};
            otherwise
                error('Ambiguous input arguments')
        end
    otherwise % case {2,4}
        switch nargin
            case 8
                FOVx = 128;
                FOVy = 128;
                interp_method = 'linear';
                SNR = [];
                w = 160;
            case 9
                FOVx = 128;
                FOVy = 128;
                interp_method = varargin{1};
                SNR = [];
                w = 160;
            case 10
                FOVx = 128;
                FOVy = 128;
                interp_method = varargin{1};
                SNR = varargin{2};
                w = 160;
            case 11
                FOVx = varargin{1};
                FOVy = varargin{2};
                interp_method = varargin{3};
                SNR = [];
                w = 160;
            case 12
                FOVx = varargin{1};
                FOVy = varargin{2};
                interp_method = varargin{3};
                SNR = varargin{4};
                w = 160;
            case 13
                FOVx = varargin{1};
                FOVy = varargin{2};
                interp_method = varargin{3};
                SNR = varargin{4};
                w = varargin{5};
            otherwise
                error('Ambiguous input arguments')
        end
end

C = (Nx/FOVx)*(Ny/FOVy)*A*(2*pi*r^2)/abs(a*b);
Nmax = max(N);
t = linspace(0, 1, Nmax^2);
%[rho, phi] = meshgrid(-N/2:N/2-1,0:pi/N:pi-pi/N);
%kx = 1/(FOVx)*(0.8*Nmax/2)*t.*cos(2*pi*w*t);
%ky = 1/(FOVy)*(0.8*Nmax/2)*t.*sin(2*pi*w*t);
kx = 1/(FOVx)*(1*Nmax/2)*t.*cos(2*pi*w*t);
ky = 1/(FOVy)*(1*Nmax/2)*t.*sin(2*pi*w*t);
rad = theta*pi/180;
kxr = 1/a*(kx*cos(rad)+ky*sin(rad));
kyr = 1/b*(-kx*sin(rad)+ky*cos(rad));
rho = sqrt(kxr.^2+kyr.^2);
f = C*jinc(2*pi*r*rho).*exp(-i*2*pi*(kx*x0+ky*y0));

if ~isempty(SNR)
    sigmakx = sqrt((Nx*Ny)*(A^2)*power(10,-SNR/10)/(2));
    N = random('Normal',0,sigmakx,prod(size(f)),1)+sqrt(-1)*random('Normal',0,sigmakx,prod(size(f)),1);
    N = reshape(N,size(f));
    f = f+N;
end

if nargout > 1
    switch nargout
        case 2
            varargout{1} = abs(Recon_arbitrarykspace(kx,ky,f,Nx,Ny,interp_method,FOVx,FOVy));
        case 3
            varargout{1} = kx;
            varargout{2} = ky;
        otherwise
            varargout{1} = abs(Recon_arbitrarykspace(kx,ky,f,Nx,Ny,interp_method,FOVx,FOVy));
            varargout{2} = kx;
            varargout{3} = ky;
    end
end