function varargout = abdom2D(datatype, Nsiz, Nsets, txy, theta, varargin)
% ABDOM2D 2-D phantom image or Fourier data of an abdomen (table lookup).
%%    datatype: 'i' for image; 'F' for k-space data
%
%    [I] = ABDOM2D('i', NSIZ, NSETS, TXY, THETA) returns an NSIZxNSIZxNSETS
%    matrix of a 2-D abdominal phantom dataset. NSIZ determines a size of
%    NSIZ(1)xNSIZ(2) matrix for each image slice. If NSIZ is not a 1x2
%    matrix, an image will have a size of NSIZxNSIZ. NSETS determines a
%    number of slices for the phantom dataset. TXY and THETA are
%    translation and rotation corresponding to each image slice. TXY is a
%    2xN matrix for which each row is corresponded to translation along x-
%    and y-axis respectively. THETA is a 1xN row vector for in-plane
%    rotation. Compartments in an image have default, constant intensity.
%    Image time frame of each slice is the image index matrix [1:Nsets].
%    The default Field-of-Views (FOVs) along x- and y-axis are both 128. No
%    Gaussian noise is added.
%
%    [I] = ABDOM2D('i', NSIZ, NSETS, TXY, THETA, FOVx, FOVy) returns an
%    NSIZ(1)xNSIZ(2)xNSETS matrix of a 2-D abdominal phantom dataset with
%    parameters as described above. FOVx and FOVy are used to determine the
%    Field-of-View (FOV) of X and Y directions respectively. Compartments
%    in an image dataset have default intensity and corresponding time
%    frame. No Gaussian noise is added.
%
%    [I] = ABDOM2D('i', NSIZ, NSETS, TXY, THETA, TIP, FOVx, FOVy) returns
%    an NSIZ(1)xNSIZ(2)xNSETS matrix of a 2-D abdominal phantom dataset
%    with parameters as described above. A temporal intensity profile (TIP)
%    of all compartments in an image dataset is determined by a 3xNSETS TIP
%    matrix. No Gaussian noise is added.
%
%    Note: If the temporal intensity profile matrix is input as an empty
%    matrix, e.g. tip =[], the routine uses a default temporal intensity
%    profile that has dynamic contrast change for a phantom dataset. This
%    option is to simulate dynamic contrast-enhanced image dataset for
%    tumor imaging study.
%
%    [I] = ABDOM2D('i', NSIZ, NSETS, TXY, THETA, TIP, TFRAME, FOVx, FOVy) 
%    returns an NSIZ(1)xNSIZ(2)xNSETS matrix of a 2-D abdominal phantom
%    dataset with parameters as described above. A temporal intensity
%    profile (TIP) with corresponding time frame for all compartments in an
%    image dataset is determined by a 3xNSETS TIP matrix and a 1xNSETS
%    TFRAME matrix respectively. No Gaussian noise is added.
%
%    All compartments allowed for temporal intensity profile are:
%       1. A circle inside the ring
%       2. The background within the ellipse, excluding the other inner
%          compartments
%       3. The ring
%
%    Note: Temporal intensity profile of each compartment corresponds to
%    each row in the 3xNSETS TIP matrix. For image dataset with default
%    dynamic contrast change, 1st compartment is the default compartment
%    that has intensity change.
%
%    [I] = ABDOM2D('i', NSIZ, NSETS, TXY, THETA, TIP, TFRAME, FOVx, FOVy, SNR)
%    returns an NSIZ(1)xNSIZ(2)xNSETS matrix of a 2-D abdominal phantom
%    dataset with parameters as described above. Gaussian noise is added to
%    produce a phantom image dataset with a given SNR (in dB), assuming
%    signal strength from averaging overall signal power in k-space data
%    generated.
%
%    [I, SIZ] = ABDOM2D('i', NSIZ, NSETS, TXY, THETA, ...) also returns a
%    row vector SIZ for a dimension matrix of a 2-D abdominal phantom
%    dataset with parameters as described above.
%
%    [F] = ABDOM2D('f', NSIZ, NSETS, TXY, THETA, ...) returns an
%    NSIZ(1)xNSIZ(2)xNSETS matrix of a 2-D k-space data, insted of an image
%    dataset, of a 2-D abdominal phantom dataset with parameters previously
%    described above.
%
%    [F, SIZ] = ABDOM2D('f', NSIZ, NSETS, TXY, THETA, ...) also returns a
%    row vector SIZ for a dimension matrix of a 2-D abdominal phantom
%    dataset with parameters as described above.
%
%    Note: The default output is assumed to return a 2-D abdominal phantom
%    image dataset.
%
%    For example, to reconstruct a 256x256x5 abdominal phantom dataset from
%    k-space data with 1 unit translation along x and y directions and 10
%    degree rotation per each image frame,
%
%        Nsets = 5;
%        Nsiz = [256 256];  % or Nsiz = 256;
%        txy = [0:Nsets-1;0:Nsets-1];
%        theta = 10*[0:Nsets-1];
%        I = abdom2D('i', Nsiz, Nsets, txy, theta);
%
%    To veiw an image, e.g. the first frame, from the phantom dataset,
%
%        imagesc(I(:,:,1)),colormap('gray')
%
%    To reconstruct the previous 256x256x5 abdominal phantom dataset with
%    the default temporal intensity profile,
%
%        Nsets = 5;
%        Nsiz = [256 256];  % or Nsiz = 256;
%        txy = [0:Nsets-1;0:Nsets-1];
%        theta = 10*[0:Nsets-1];
%        tip = [];
%        FOVx = 128;
%        FOVy = 128;
%        I = abdom2D('i', Nsiz, Nsets, txy, theta, tip, FOVx,FOVy);  
%
%    The temporal intensity profile matrix can be manually determined in a
%    form of a 9xNSets matrix.
%
%    See also CHEST2D, DCEK, FCIRC, and FBOX.
%
%    Copyrigt 2007, Jim Ji, Texas A&M University. All Rights Reserved. 
%

error(nargchk(5, 10, nargin))
error(nargoutchk(1, 2, nargout));
if ~isstr(datatype)
    error('DATATYPE must be string')
end

switch nargin
    case 5
        FOVx = 128;
        FOVy = 128;
        tframe = [1:Nsets];
        tip = [30 192 255]'*ones(1,Nsets);
        SNR = [];        
    case 6
        error('Ambiguous input arguments')
    case 7
        FOVx = varargin{1};
        FOVy = varargin{2};
        tframe = [1:Nsets];
        tip = [30 192 255]'*ones(1,Nsets);
        SNR = [];
    case 8
        FOVx = varargin{2};
        FOVy = varargin{3};
        tframe = [1:Nsets];
        tip = varargin{1};
        SNR = [];
    case 9
        FOVx = varargin{3};
        FOVy = varargin{4};
        tframe = varargin{2};
        tip = varargin{1};
        SNR = [];
    otherwise
        FOVx = varargin{3};
        FOVy = varargin{4};
        tframe = varargin{2};
        tip = varargin{1};
        SNR = varargin{5};
end

switch length(Nsiz)
    case 1
        Nx = Nsiz;
        Ny = Nsiz;
    case 2
        Nx = Nsiz(1);
        Ny = Nsiz(2);
    otherwise
        error('Invalid dimension: Nsiz')
end
I = zeros(Ny,Nx,Nsets);

if isempty(tip)
    tframe = 0:120:120*(Nsets-1);
    CA = tcac(tframe);
    RI = c2i(CA);
    tip = [30 192 255]'*ones(1,Nsets);
    tip(1,:) = 240*RI(4,:);
end

rad = theta*pi/180;
txy = [FOVx*txy(1,:)/Nx; FOVy*txy(2,:)/Ny];

scale = [0.75; 1.4];
[fc1] = dcek('f', tframe, tip(3,:), theta, txy, 'circle', 35.3, Nsiz, FOVx, FOVy, scale(1), scale(2));
[fc2] = dcek('f', tframe, tip(3,:)-tip(2,:), theta, txy, 'circle', 33.7, Nsiz, FOVx, FOVy, scale(1), scale(2));

R = [cos(rad(:)) -sin(rad(:)); sin(rad(:)) cos(rad(:))];
xy = R*[15;0];
xy = reshape(xy,Nsets,2)';
[fc3] = dcek('f', tframe, tip(1,:)-tip(2,:), theta, xy+txy, 'circle', 10.3, Nsiz, FOVx, FOVy);

f = (fc1-fc2+fc3);

if ~isempty(SNR)
    f = f+addnoise(f,SNR);
end

switch upper(datatype)
    case 'F'
        if nargout == 2
            varargout{1} = f;
            varargout{2} = size(f);
        else
            varargout{1} = f;
        end
    otherwise
        for i = 1:Nsets
            I(:,:,i) = abs(ifftshift(ifft2(ifftshift(squeeze(f(:,:,i))))));
        end
        if nargout == 2
            varargout{1} = I;
            varargout{2} = size(I);
        else
            varargout{1} = I;
        end
end