function varargout = brain2D(datatype, Nsiz, Nsets, txy, theta, varargin)
% BRAIN2D 2-D phantom image or Fourier data of a brain (table lookup).
%%    datatype: 'i' for image; 'F' for k-space data
%    [I] = BRAIN2D('i', NSIZ, NSETS, TXY, THETA) returns an NSIZxNSIZxNSETS
%    matrix of a 2-D brain phantom dataset. NSIZ determines a size of
%    NSIZ(1)xNSIZ(2) matrix for each image slice. If NSIZ is not a 1x2
%    matrix, an image will have a size of NSIZxNSIZ. NSETS determines a
%    number of slices for the phantom dataset. TXY and THETA are
%    translation and rotation corresponding to each image slice. TXY is a
%    2xNSETS matrix for which each row is corresponded to translation along
%    x- and y-axis respectively. THETA is a 1xNSETS row vector for in-plane
%    rotation. Compartments in an image have default, constant intensity.
%    Image time frame of each slice is the image index matrix [1:Nsets].
%    The default Field-of-Views (FOVs) along x- and y-axis are both 128. No
%    Gaussian noise is added.
%
%    [F] = BRAIN2D('f', NSIZ, NSETS, TXY, THETA, ...) returns an
%    NSIZ(1)xNSIZ(2)xNSETS matrix of a 2-D k-space data, insted of an image
%    dataset, of a 2-D brain phantom dataset with parameters previously
%    described above.
%
%    [I] = BRAIN2D('i', NSIZ, NSETS, TXY, THETA, FOVx, FOVy) returns an
%    NSIZ(1)xNSIZ(2)xNSETS matrix of a 2-D brain phantom dataset with
%    parameters as described above. FOVx and FOVy are used to determine the
%    Field-of-View (FOV) along X and Y directions respectively.
%    Compartments in an image dataset have default intensity and
%    corresponding time frame. No Gaussian noise is added.
%
%    [I] = BRAIN2D('i', NSIZ, NSETS, TXY, THETA, TIP, FOVx, FOVy) returns
%    an NSIZ(1)xNSIZ(2)xNSETS matrix of a 2-D brain phantom dataset with
%    parameters as described above. A temporal intensity profile (TIP)of
%    all compartments in an image dataset is determined by a 9xNSETS TIP
%    matrix. No Gaussian noise is added.
%
%    Note: If the temporal intensity profile matrix is input as an empty
%    matrix, e.g. tip =[], the routine uses a default temporal intensity
%    profile that has dynamic contrast change for a phantom dataset. This
%    option is to simulate dynamic contrast-enhanced image dataset for
%    tumor imaging study.
%
%    [I] = BRAIN2D('i', NSIZ, NSETS, TXY, THETA, TIP, TFRAME, FOVx, FOVy) 
%    returns an NSIZ(1)xNSIZ(2)xNSETS matrix of a 2-D brain phantom dataset
%    with parameters as described above. A temporal intensity profile (TIP)
%    with corresponding time frame for all compartments in an image dataset
%    is determined by a 9xNSETS TIP matrix and a 1xNSETS TFRAME matrix
%    respectively. No Gaussian noise is added.
%
%    All compartments allowed for temporal intensity profile are:
%       1. An upper-left rectangle
%       2. An upper-center rectangle
%       3. An upper-right rectangle
%       4. A right ellipse
%       5. A left ellipse
%       6. The first lower rectangle
%       7. The second lower rectangle
%       8. The third lower rectangle
%       9. The big ellipse containing all compartments inside
%
%    Note: Temporal intensity profile of each compartment corresponds to
%    each row in the 9xNSETS TIP matrix. For image dataset with default
%    dynamic contrast change, 1st, 3rd, 4th, 6th, and 7th compartments are
%    default compartments that have intensity change.
%
%    [I] = BRAIN2D('i', NSIZ, NSETS, TXY, THETA, TIP, TFRAME, FOVx, FOVy, SNR)
%    returns an NSIZ(1)xNSIZ(2)xNSETS matrix of a 2-D brain phantom dataset
%    with parameters as described above. Gaussian noise is added to produce
%    a phantom image dataset with a given SNR (in dB), assuming signal
%    strength from averaging overall signal power in k-space data generated.
%
%    [I] = BRAIN2D('i', NSIZ, NSETS, TXY, THETA, TIP, TFRAME, FOVx, FOVy, SNR, FILTER)
%    returns an NSIZ(1)xNSIZ(2)xNSETS matrix of a 2-D brain phantom dataset
%    with parameters as described above. Gaussian noise is added to produce
%    a phantom image dataset with a given SNR (in dB), assuming signal
%    strength from averaging overall signal power in k-space data
%    generated. FILTER is used to apply a 2-D hamming window on the k-space
%    data before image reconstruction, resulting in smoothed images. FILTER
%    needs to be non-empty matrix to identify the request of using a
%    hamming window.
%
%    [I, SIZ] = BRAIN2D('i', NSIZ, NSETS, TXY, THETA, ...) also returns a
%    row vector SIZ for a dimension matrix of a 2-D brain phantom dataset
%    with parameters as described above.
%

%
%    [F, SIZ] = BRAIN2D('f', NSIZ, NSETS, TXY, THETA, ...) also returns a
%    row vector SIZ for a dimension matrix of a 2-D brain phantom dataset
%    with parameters as described above.
%
%    Note: The default output is assumed to return a 2-D brain phantom
%    image dataset.
%
%    For example, to reconstruct a 256x256x10 brain phantom dataset from
%    k-space data with 1 unit translation along x and y directions and 10
%    degree rotation per each image frame,
%
%        Nsets = 10;
%        Nsiz = [256 256];  % or Nsiz = 256;
%        t = linspace(0,1,Nsets);    
%        txy = 0.7*[0:Nsets-1;0:Nsets-1];
%        theta = 45*[sin(2*pi*t)];
%        I = brain2D('i', Nsiz, Nsets, txy, theta);
%
%    To veiw an image, e.g. the first frame, from the phantom dataset,
%
%        imagesc(I(:,:,1)),colormap('gray')
%
%    To reconstruct the previous 256x256x10 brain phantom dataset with the
%    default temporal intensity profile,
%
%        Nsets = 10;
%        Nsiz = [256 256];  % or Nsiz = 256;
%        t = linspace(0,1,Nsets);    
%        txy = 0.7*[0:Nsets-1;0:Nsets-1];
%        theta = 45*[sin(2*pi*t)];
%        tip = [];
%        FOVx = 128;
%        FOVy = 128;
%        I = brain2D('i', Nsiz, Nsets, txy, theta, tip, FOVx,FOVy);  
%
%    The temporal intensity profile matrix can be manually determined in a
%    form of a 9xNSets matrix.
%
%    See also SHEPPLOGAN, DCEK, FCIRC, and FBOX.
%    Created and modified by Yuttapong Jiraraksopakun
%
%
%    Copyrigt 2007, Jim Ji, Texas A&M University. All Rights Reserved. 
%

error(nargchk(5, 11, nargin))
error(nargoutchk(1, 2, nargout));
if ~isstr(datatype)
    error('DATATYPE must be string')
end

switch nargin
    case 5
        FOVx = 128;
        FOVy = 128;
        tframe = [1:Nsets];
        tip = [60 60 60 90 90 230 60 60 60 192]'*ones(1,Nsets);
        SNR = [];
        filter = [];
    case 6
        error('Ambiguous input arguments')
    case 7
        FOVx = varargin{1};
        FOVy = varargin{2};
        tframe = [1:Nsets];
        tip = [60 60 60 90 90 230 60 60 60 192]'*ones(1,Nsets);
        SNR = [];
        filter = [];
    case 8
        FOVx = varargin{2};
        FOVy = varargin{3};
        tframe = [1:Nsets];
        tip = varargin{1};
        SNR = [];
        filter = [];
    case 9
        FOVx = varargin{3};
        FOVy = varargin{4};
        tframe = varargin{2};
        tip = varargin{1};
        SNR = [];
        filter = [];
    case 10
        FOVx = varargin{3};
        FOVy = varargin{4};
        tframe = varargin{2};
        tip = varargin{1};
        SNR = varargin{5};
        filter = [];
    otherwise
        FOVx = varargin{3};
        FOVy = varargin{4};
        tframe = varargin{2};
        tip = varargin{1};
        SNR = varargin{5};
        filter = varargin{6};
end

switch length(Nsiz)
    case 1
        Nx = Nsiz;
        Ny = Nsiz;
    case 2
        Nx = Nsiz(1);
        Ny = Nsiz(2);
    otherwise
        error('Invalid dimension: Nsiz')
end
I = zeros(Ny,Nx,Nsets);

if isempty(tip)
    tframe = 0:120:120*(Nsets-1);
    CA = tcac(tframe);
    RI = c2i(CA);
    tip = [60 60 60 90 90 230 60 60 60 192]'*ones(1,Nsets);
%     tip(1,:) = tip(1,:);
%     tip(2,:) = tip(2,:);
%     tip(3,:) = tip(3,:);
    tip(4,:) = tip(4,1)+7*[0:Nsets-1];
    tip(5,:) = tip(5,1)+7*[0:Nsets-1];
    tip(6,:) = 240*RI(4,:);
%     tip(7,:) = tip(7,:);
%     tip(8,:) = tip(8,:);
%     tip(9,:) = tip(9,:);
end

rad = theta*pi/180;
txy = [FOVx*txy(1,:)/Nx; FOVy*txy(2,:)/Ny];

%[fc1] = dcek('f', tframe, tip(10,:), theta, txy, 'ellipse', 45.7, 1.2, 0.75, Nsiz, FOVx, FOVy, SNR);
[fc1] = dcek('f', tframe, tip(10,:), theta, txy, 'ellipse', 45.7, 0.85, 0.8, Nsiz, FOVx, FOVy, SNR);

R = [cos(rad(:)) -sin(rad(:)); sin(rad(:)) cos(rad(:))];
xy = R*[-5;42.7];
xy = reshape(xy,Nsets,2)';
[fb1] = dcek('f', tframe, tip(1,:)-tip(10,:), theta, xy+txy, 'box', 1.35, 12.7, Nsiz, FOVx, FOVy, SNR);

xy = R*[0;42.7];
xy = reshape(xy,Nsets,2)';
[fb2] = dcek('f', tframe, tip(2,:)-tip(10,:), theta, xy+txy, 'box', 1.35, 12.7, Nsiz, FOVx, FOVy, SNR);

xy = R*[5;42.7];
xy = reshape(xy,Nsets,2)';
[fb3] = dcek('f', tframe, tip(3,:)-tip(10,:), theta, xy+txy, 'box', 1.35, 12.7, Nsiz, FOVx, FOVy, SNR);

xy = R*[25;0.4];
xy = reshape(xy,Nsets,2)';
%[fc2] = dcek('f', tframe, tip(4,:)-tip(10,:), theta-7, xy+txy, 'ellipse', 16.7, 2.3, 0.6, Nsiz, FOVx, FOVy, SNR);
[fc2] = dcek('f', tframe, tip(4,:)-tip(10,:), theta-7, xy+txy, 'ellipse', 25.3, 2.55, 0.6, Nsiz, FOVx, FOVy, SNR);

xy = R*[-25;0.4];
xy = reshape(xy,Nsets,2)';
[fc3] = dcek('f', tframe, tip(5,:)-tip(10,:), theta+7, xy+txy, 'ellipse', 25.3, 2.55, 0.6, Nsiz, FOVx, FOVy, SNR);

xy = R*[0;14.3];
xy = reshape(xy,Nsets,2)';
[fc4] = dcek('f', tframe, tip(6,:)-tip(10,:), theta, xy+txy, 'ellipse', 15.3, 1, 1, Nsiz, FOVx, FOVy, SNR);

xy = R*[0;-40.7];
xy = reshape(xy,Nsets,2)';
[fb4] = dcek('f', tframe, tip(7,:)-tip(10,:), theta, xy+txy, 'box', 12.7, 1.35, Nsiz, FOVx, FOVy, SNR);

xy = R*[0;-45.7];
xy = reshape(xy,Nsets,2)';
[fb5] = dcek('f', tframe, tip(8,:)-tip(10,:), theta, xy+txy, 'box', 12.7, 1.35, Nsiz, FOVx, FOVy, SNR);

xy = R*[0;-50.7];
xy = reshape(xy,Nsets,2)';
[fb6] = dcek('f', tframe, tip(9,:)-tip(10,:), theta, xy+txy, 'box', 12.7, 1.35, Nsiz, FOVx, FOVy,  SNR);

f = fc1+fc2+fc3+fc4+fb1+fb2+fb3+fb4+fb5+fb6;

if ~isempty(filter)
    winfunc = str2func('hamming');
    if mod(Nx,2) == 0
        wx = window(winfunc,Nx+1);
    else
        wx = window(winfunc,Nx);
    end
    
    if mod(Ny,2) == 0
        wy = window(winfunc,Ny+1);
    else
        wy = window(winfunc,Ny);
    end
    w = repmat(wy(1:end-1)*wx(1:end-1)',[1 1 Nsets]);
    f = f.*w;
end

% if ~isempty(SNR)
%     f = f+addnoise(f,SNR);
% end

switch upper(datatype)
    case 'F'
        if nargout == 2
            varargout{1} = f;
            varargout{2} = size(f);
        else
            varargout {1}= f;
        end
    otherwise
        for i = 1:Nsets
            I(:,:,i) = ifftshift(ifft2(ifftshift(squeeze(f(:,:,i)))));
        end
        if nargout == 2
            varargout{1} = I;
            varargout{2} = size(I);
        else
            varargout{1} = I;
        end
end