function [outdata, varargout] = dcek(datatype, tframe, tip, theta, dxy, shape, varargin)
% DCEK 2-D image or Fourier data of a simple structure (table lookup).
% Dynamic Contrast-Enhanced K-space data routine.
%
%    [I] = DCEK('i', TFRAME, TIP, THETA, DXY, SHAPE, ...) returns an
%    NSIZ(1)xNSIZ(2)xNSETS matrix of 2-D image of a structure specified by
%    SHAPE with a 1xNSETS row vector TIP of temporal intensity profile.
%    TFRAME is a 1xNSETS row vector of time corresponding to the temporal
%    intensity profile matrix. DXY and THETA are translation and rotation
%    corresponding to the temporal intensity profile and time frame. DXY is
%    a 2xNSETS matrix for translation along x- and y-axis respectively.
%    THETA is a 1xNSETS row vector for in-plane rotation.    
%
%    [I,T] = DCEK('i', TFRAME, TIP, THETA, DXY, SHAPE, ...) also returns a
%    1xNSETS row vector TFRAME for time frame corresponding to temporal
%    intensity profile.
%
%    [F] = DCEK('f', TFRAME, TIP, THETA, DXY, SHAPE, ...) returns an
%    NSIZ(1)xNSIZ(2)xNSETS matrix of 2-D k-space data, insted of an image,
%    of a structure specified by SHAPE with parameters previously described
%    above.
%
%    [F,T] = DCEK('f', TFRAME, TIP, THETA, DXY, SHAPE, ...) also returns a
%    1xNSETS row vector TFRAME for time frame corresponding to temporal
%    intensity profile.
%
%    Note: The default output is assumed to return a 2-D image of a
%    specified shape.
%
%    Available shapes are such as:
%
%           'ellipse' - an ellipse with radius R and scaling a and b
%           'box' - a rectangle with height A and width B
%
%    Note: A circle can be obtained from an ellipse with unity scale on
%    both X and Y directions.
%
%    Size of an image must also be determined for a speicifed shape for
%    Fellipse and Fbox. Scaling and field-of-view (FOV) can also be
%    determined.
%
%    Note: The intensity (A), translation (dx,dy) and in-plane rotation
%    (theta) parameters should be excluded when Fellipse and Fbox routines
%    are referred for required parameters according to a determined shape.
%
%    For example, to reconstruct a 256x256x5 circle image from k-space data
%    with initial 100 intensity units and radius 30 units, 5 units
%    translation along x and y directions, 10 degree rotation, and 20
%    intensity units increase per image frame,
%
%        Nsets = 5;
%        Nsiz = [256 256]; %or Nsiz = 256;
%        tframe = [1:Nsets];
%        tip = 100+20*[1:Nsets];
%        theta = 10*[0:Nsets-1];
%        dxy = 5*[0:Nsets-1];
%        FOVx = 128;
%        FOVy = 128;
%        I = dcek('i', tframe, tip, theta, dxy, 'ellipse', 30, 1, 1, Nsiz, FOVx, FOVy);
%
%    To veiw an image, e.g. the first frame, from the image dataset,
%
%        imagesc(I(:,:,1)),colormap('gray')
%
%    To reconstruct a 256x256x5 rectangle image from k-space data with
%    height 40 units, width 30 units and the same other parameters as the
%    previous example,
%
%        Nsets = 5;
%        Nsiz = [256 256]; %or Nsiz = 256;
%        tframe = [1:Nsets];
%        tip = 100+20*[1:Nsets];
%        theta = 10*[0:Nsets-1];
%        dxy = 5*[0:Nsets-1];
%        FOVx = 128;
%        FOVy = 128;
%        I = dcek('i', tframe, tip, theta, dxy, 'box', 30, 40, Nsiz, FOVx, FOVy);
%
%    See also SHEPPLOGAN, BRAIN2D, FCIRC, and FBOX.
%
%    Created and modified by Yuttapong Jiraraksopakun
%
%    Copyrigt 2007, Jim Ji, Texas A&M University. All Rights Reserved. 
%


error(nargoutchk(1, 2, nargout));
if ~isstr(datatype)
    error('DATATYPE must be string')
end
if ~isstr(shape)
    error('SHAPE must be string')
end
sizechk = [size(tframe);size(tip);size(theta);size(dxy)];
if ~isequal(sizechk(1:3,1),[1 1 1]') | ~isequal(sizechk(:,2), length(tframe)*[1 1 1 1]') | (sizechk(4,1) ~= 2)
    error('Dimension mismatched: check tframe, tint, theta, or dxy')
end

switch lower(shape(1))
    case 'e'
        %Fellipse(intensity,r,a,b,theta,dx,dy,Nsiz,FOVx,FOVy,SNR);
        for ii = 1:length(tip)
            F(:,:,ii) = Fellipse(tip(ii),varargin{1:3},theta(ii),dxy(1,ii),dxy(2,ii),varargin{4:end});
        end
    case 'b'
        %Fbox(intensity,a,b,theta,dx,dy,Nsiz,FOVx,FOVy,SNR,Sx,Sy);
        for ii = 1:length(tip)
            F(:,:,ii) = Fbox(tip(ii),varargin{1:2},theta(ii),dxy(1,ii),dxy(2,ii),varargin{3:end});
        end
    otherwise
        error('Invalid shape')
end

switch lower(datatype(1))
    case 'f'
        outdata = F;
    otherwise % 'i'
        for ii = 1:length(tint)
            outdata(:,:,ii) = abs(ifftshift(ifft2(ifftshift(squeeze(F(:,:,ii))))));
        end
end
if nargout > 1
    varargout{1} = tframe;
end