function [outdata, varargout] = dcekr(datatype, tframe, tip, theta, dxy, shape, varargin)
% DCEKR 2-D image or radial Fourier data of a simple structure with radial trajectory (table lookup).
% Dynamic Contrast-Enhanced K-space data routine.
%
%    [I] = DCEKR('i', TFRAME, TIP, THETA, DXY, SHAPE, ..., INT_METHOD, FILTER)
%    returns an NSIZ(1)xNSIZ(2)xNSETS matrix of 2-D image of a structure
%    specified by SHAPE with a 1xNSETS row vector TIP of temporal intensity
%    profile. TFRAME is a 1xNSETS row vector of time corresponding to the
%    temporal intensity profile matrix. DXY and THETA are translation and
%    rotation corresponding to the temporal intensity profile and time
%    frame. DXY is a 2xNSETS matrix for translation along x- and y-axis
%    respectively. THETA is a 1xNSETS row vector for in-plane rotation.
%
%    Note: DCEKR only uses FBOXR or FELLIPSER to return the k-space data or
%    the k-space data with its coordinates. Hence, variable input arguments
%    for DCEKR need to be consistent with variable input arguments of those
%    routines to return the k-space data or the k-space data with its
%    coordinates only. See the examples below for details.
%
%    [I,T] = DCEKR('i', TFRAME, TIP, THETA, DXY, SHAPE, ..., INT_METHOD, FILTER)
%    also returns a 1xNSETS row vector TFRAME for time frame corresponding
%    to temporal intensity profile.
%
%    [F] = DCEKR('f', TFRAME, TIP, THETA, DXY, SHAPE, ...) returns an
%    NSIZ(1)xNSIZ(2)xNSETS matrix of 2-D k-space data, insted of an image,
%    of a structure specified by SHAPE with parameters previously described
%    above.
%
%    [F,T] = DCEKR('f', TFRAME, TIP, THETA, DXY, SHAPE, ...) also returns a
%    1xNSETS row vector TFRAME for time frame corresponding to temporal
%    intensity profile.
%
%    Note: The default output is assumed to return a 2-D image of a
%    specified shape.
%
%    Available shapes are such as:
%
%           'ellipse' - an ellipse with radius R and scaling a and b
%           'box' - a rectangle with height A and width B
%
%    Note: A circle can be obtained from an ellipse with unity scale on
%    both X and Y directions.
%
%    Size of an image must also be determined for a speicifed shape for
%    Fcirc and Fbox. Scaling and field-of-view (FOV) can also be determined.
%
%    Note: The intensity (A), translation (dx,dy) and in-plane rotation
%    (theta) parameters should be excluded when Fcirc and Fbox routines are
%    referred for required parameters according to a determined shape.
%
%    For example, to reconstruct a 256x256x5 circle image from k-space data
%    with initial 100 intensity units and radius 30 units, 5 units
%    translation along x and y directions, 10 degree rotation, and 20
%    intensity units increase per image frame,
%
%        Nsets = 5;
%        Nsiz = [256 256]; %or Nsiz = 256;
%        tframe = [1:Nsets];
%        tip = 100+20*[1:Nsets];
%        theta = 10*[0:Nsets-1];
%        dxy = 5*[0:Nsets-1];
%        FOVx = 128;
%        FOVy = 128;
%        I = dcekr('i', tframe, tip, theta, dxy, 'circle', 30, Nsiz, FOVx, FOVy);
%
%    To veiw an image, e.g. the first frame, from the image dataset,
%
%        imagesc(I(:,:,1)),colormap('gray')
%
%    To reconstruct a 256x256x5 rectangle image from k-space data with
%    height 40 units, width 30 units and the same other parameters as the
%    previous example,
%
%        Nsets = 5;
%        Nsiz = [256 256]; %or Nsiz = 256;
%        tframe = [1:Nsets];
%        tip = 100+20*[1:Nsets];
%        theta = 10*[0:Nsets-1];
%        dxy = 5*[0:Nsets-1;0:Nsets-1];
%        FOVx = 128;
%        FOVy = 128;
%        I = dcekr('i', tframe, tip, theta, dxy, 'box', 30, 40, Nsiz, FOVx, FOVy);
%
%    See also SLRADIAL, FCIRCR, and FBOXR.
%
%    Created and modified by Yuttapong Jiraraksopakun
%
%
%    Copyrigt 2007, Jim Ji, Texas A&M University. All Rights Reserved. 
%

error(nargoutchk(1, 4, nargout));
if ~isstr(datatype)
    error('DATATYPE must be string')
end
if ~isstr(shape)
    error('SHAPE must be string')
end
sizechk = [size(tframe);size(tip);size(theta);size(dxy)];
if ~isequal(sizechk(1:3,1),[1 1 1]') | ~isequal(sizechk(:,2), length(tframe)*[1 1 1 1]') | (sizechk(4,1) ~= 2)
    error('Dimension mismatched: check tframe, tint, theta, or dxy')
end

switch lower(datatype(1))
    case 'f'
        switch lower(shape(1))
            case 'e'
                %Fellipser(intensity,r,a,b,theta,dx,dy,Nsiz,FOVx,FOVy,SNR);
                Nsiz = varargin{2};
                if nargout > 2
                    [F(:,:,1), kx, ky] = Fellipser(tip(1),varargin{1:3},theta(1),dxy(1,1),dxy(2,1),varargin{4:end});
                else
                    F(:,:,1) = Fellipser(tip(1),varargin{1:3},theta(1),dxy(1,1),dxy(2,1),varargin{4:end});
                end
                for ii = 2:length(tip)
                    F(:,:,ii) = Fellipser(tip(ii),varargin{1:3},theta(ii),dxy(1,ii),dxy(2,ii),varargin{4:end});
                end
            case 'b'
                %Fbox(intensity,a,b,theta,dx,dy,Nsiz,FOVx,FOVy,SNR,Sx,Sy);
                Nsiz = varargin{3};
                if nargout > 2
                    [F(:,:,1), kx, ky] = Fboxr(tip(1),varargin{1:2},theta(1),dxy(1,1),dxy(2,1),varargin{3:end});
                else
                    F(:,:,1) = Fboxr(tip(1),varargin{1:2},theta(1),dxy(1,1),dxy(2,1),varargin{3:end});
                end
                for ii = 2:length(tip)
                    F(:,:,ii) = Fboxr(tip(ii),varargin{1:2},theta(ii),dxy(1,ii),dxy(2,ii),varargin{3:end});
                end
            otherwise
                error('Invalid shape')
        end
        outdata = F;
    otherwise % 'i'
        switch lower(shape(1))
            case 'e'
                %Fellipser(intensity,r,a,b,theta,dx,dy,Nsiz,FOVx,FOVy,SNR);
                Nsiz = varargin{2};
                if nargout > 2
                    [F(:,:,1), kx, ky] = Fellipser(tip(1),varargin{1:3},theta(1),dxy(1,1),dxy(2,1),varargin{4:end-2});
                else
                    F(:,:,1) = Fellipser(tip(1),varargin{1:3},theta(1),dxy(1,1),dxy(2,1),varargin{4:end-2});
                end
                for ii = 2:length(tip)
                    F(:,:,ii) = Fellipser(tip(ii),varargin{1:3},theta(ii),dxy(1,ii),dxy(2,ii),varargin{4:end-2});
                end
            case 'b'
                %Fbox(intensity,a,b,theta,dx,dy,Nsiz,FOVx,FOVy,SNR,Sx,Sy);
                Nsiz = varargin{3};
                if nargout > 2
                    [F(:,:,1), kx, ky] = Fboxr(tip(1),varargin{1:2},theta(1),dxy(1,1),dxy(2,1),varargin{3:end-2});
                else
                    F(:,:,1) = Fboxr(tip(1),varargin{1:2},theta(1),dxy(1,1),dxy(2,1),varargin{3:end-2});
                end
                for ii = 2:length(tip)
                    F(:,:,ii) = Fboxr(tip(ii),varargin{1:2},theta(ii),dxy(1,ii),dxy(2,ii),varargin{3:end-2});
                end
            otherwise
                error('Invalid shape')
        end
        for ii = 1:length(tint)
%             outdata(:,:,ii) = abs(Recon_arbitrarykspace(kx,ky,squeeze(F(:,:,ii)),Nsiz(1),Nsiz(2),'linear',FOVx,FOVy));
            Nmax = max(Nsiz);
            outdata(:,:,ii) = abs(fbprec( F(:,:,ii), [0:Nmax-1]*180/Nmax, Nmax, varargin{end-1:end} ));
        end
end
if nargout > 1
    switch nargout
        case 2
            varargout{1} = tframe;
        case 3
            varargout{1} = kx;
            varargout{2} = ky;
        otherwise            
            varargout{1} = tframe;
            varargout{2} = kx;
            varargout{3} = ky;
    end
end