function varargout = slradial(datatype, Nsiz, Nsets, txy, theta, varargin)
% SLRADIAL 2-D phantom image or Fourier data of a brain with radial trajectory (table lookup).
%
%    [I] = SLRADIAL('i', NSIZ, NSETS, TXY, THETA) returns an
%    NSIZ(1)xNSIZ(2)xNSETS matrix of a 2-D brain phantom dataset. NSIZ
%    determines a size of NSIZ(1)xNSIZ(2) matrix for each image slice. If
%    NSIZ is not a 1x2 matrix, an image will have a size of NSIZxNSIZ.
%    NSETS determines a number of slices for the phantom dataset. TXY and
%    THETA are translation and rotation corresponding to each image slice.
%    TXY is a 2xNSETS matrix for which each row is corresponded to
%    translation along x- and y-axis respectively. THETA is a 1xNSETS row
%    vector for in-plane rotation. Compartments in an image have default,
%    constant intensity. Image time frame of each slice is the image index
%    matrix [1:Nsets]. The default Field-of-Views (FOVs) along x- and
%    y-axis are both 128. No Gaussian noise is added. Images are obtianed
%    by the filtered back projection, or inverse Radon transform. Linear
%    interpolation and Ram-Lak filter are used as default interpolation
%    method and filter respectively for inverse Radon transform.
%
%    [I] = SLRADIAL('i', NSIZ, NSETS, TXY, THETA, FOVx, FOVy) returns an
%    NSIZ(1)xNSIZ(2)xNSETS matrix of a 2-D brain phantom dataset with
%    parameters as described above. FOVx and FOVy are used to determine the
%    Field-of-View (FOV) along X and Y directions respectively.
%    Compartments in an image dataset have default intensity and
%    corresponding time frame. No Gaussian noise is added. Linear
%    interpolation and Ram-Lak filter are used for inverse Radon transform.
%
%    [I] = SLRADIAL('i', NSIZ, NSETS, TXY, THETA, TIP, FOVx, FOVy) returns
%    an NSIZ(1)xNSIZ(2)xNSETS matrix of a 2-D brain phantom dataset with
%    parameters as described above. A temporal intensity profile (TIP)of
%    all compartments in an image dataset is determined by a 9xNSETS TIP
%    matrix. No Gaussian noise is added. Linear interpolation and Ram-Lak
%    filter are used for inverse Radon transform.
%
%    Note: If the temporal intensity profile matrix is input as an empty
%    matrix, e.g. tip =[], the routine uses a default temporal intensity
%    profile that has dynamic contrast change for a phantom dataset. This
%    option is to simulate dynamic contrast-enhanced image dataset for
%    tumor imaging study.
%
%    [I] = SLRADIAL('i', NSIZ, NSETS, TXY, THETA, TIP, TFRAME, FOVx, FOVy) 
%    returns an NSIZ(1)xNSIZ(2)xNSETS matrix of a 2-D brain phantom dataset
%    with parameters as described above. A temporal intensity profile (TIP)
%    with corresponding time frame for all compartments in an image dataset
%    is determined by a 9xNSETS TIP matrix and a 1xNSETS TFRAME matrix
%    respectively. No Gaussian noise is added. Linear interpolation and
%    Ram-Lak filter are used for inverse Radon transform.
%
%    All compartments allowed for temporal intensity profile are:
%       1. An ellipse above the two big ellipses
%       2. Two small ellipses between the two big ellipses
%       3. A big, right ellipse
%       4. A big, left ellipse
%       5. A small ellipse below the two big ellipses
%       6. A small, right ellipse
%       7. A small, left ellipse
%       8. The background within the ring, excluding the other inner
%          compartments
%       9. The elliptical ring
%
%    Note: Temporal intensity profile of each compartment corresponds to
%    each row in the 9xNSETS TIP matrix. For image dataset with default
%    dynamic contrast change, 1st, 3rd, 4th, 6th, and 7th compartments are
%    default compartments that have intensity change.
%
%    [I] = SLRADIAL('i', NSIZ, NSETS, TXY, THETA, TIP, TFRAME, FOVx, FOVy, SNR)
%    returns an NSIZ(1)xNSIZ(2)xNSETS matrix of a 2-D brain phantom dataset
%    with parameters as described above. Gaussian noise is added to produce
%    a phantom image dataset with a given SNR (in dB), assuming signal
%    strength from averaging overall signal power in k-space data
%    generated. Linear interpolation and Ram-Lak filter are used for
%    inverse Radon transform.
%
%    [I] = SLRADIAL('i', NSIZ, NSETS, TXY, THETA, TIP, TFRAME, FOVx, FOVy, SNR, INT_METHOD, FILTER)
%    returns an NSIZ(1)xNSIZ(2)xNSETS matrix of a 2-D brain phantom dataset
%    with parameters as described above. Gaussian noise is added to produce
%    a phantom image dataset with a given SNR (in dB), assuming signal
%    strength from averaging overall signal power in k-space data
%    generated. INT_METHOD and FILTER are interpolation method and filter
%    used for inverse Radon transform.
%
%    [I, SIZ] = SLRADIAL('i', NSIZ, NSETS, TXY, THETA, ...) also returns a
%    row vector SIZ for a dimension matrix of a 2-D brain phantom dataset
%    with parameters as described above.
%
%    [F] = SLRADIAL('f', NSIZ, NSETS, TXY, THETA, ...) returns an
%    NSIZ(1)xNSIZ(2)xNSETS matrix of a 2-D k-space data, insted of an image
%    dataset, of a 2-D brain phantom dataset with parameters previously
%    described above.
%
%    [F, SIZ] = SLRADIAL('f', NSIZ, NSETS, TXY, THETA, ...) also returns a
%    row vector SIZ for a dimension matrix of a 2-D brain phantom dataset
%    with parameters as described above.
%
%    Note: The default output is assumed to return a 2-D brain phantom
%    image dataset.
%
%    For example, to reconstruct a 256x256x10 brain phantom dataset from
%    k-space data with 1 unit translation along x and y directions and 10
%    degree rotation per each image frame,
%
%        Nsets = 10;
%        Nsiz = [256 256];  % or Nsiz = 256;
%        txy = 0.7*[0:Nsets-1;0:Nsets-1];
%        t = linspace(0,1,Nsets);
%        theta = 45*[sin(2*pi*t)];
%        I = slradial('i', Nsiz, Nsets, txy, theta);
%
%    To veiw an image, e.g. the first frame, from the phantom dataset,
%
%        imagesc(I(:,:,1)),colormap('gray')
%
%    To reconstruct the previous 256x256x10 brain phantom dataset with the
%    default temporal intensity profile,
%
%        Nsets = 10;
%        Nsiz = [256 256];  % or Nsiz = 256;
%        txy = 0.7*[0:Nsets-1;0:Nsets-1];
%        t = linspace(0,1,Nsets);
%        theta = 45*[sin(2*pi*t)];
%        tip = [];
%        FOVx = 128;
%        FOVy = 128;
%        I = slradial('i', Nsiz, Nsets, txy, theta, tip, FOVx,FOVy);  
%
%    The temporal intensity profile matrix can be manually determined in a
%    form of a 9xNSets matrix.
%
%    See also BRAIN2D, SHEPPLOGAN, DCEK, DCEKR, FCIRCR, and FBOXR.
%
%    Created and modified by Yuttapong Jiraraksopakun
%
%
%    Copyrigt 2007, Jim Ji, Texas A&M University. All Rights Reserved. 
%

error(nargchk(5, 12, nargin))
error(nargoutchk(1, 2, nargout));
if ~isstr(datatype)
    error('DATATYPE must be string')
end

switch nargin
    case 5
        FOVx = 128;
        FOVy = 128;
        tframe = [1:Nsets];
        tip = [30 50 60 60 100 20 20 192 255]'*ones(1,Nsets);
        SNR = [];
        interp_method = 'linear';
        filter = 'Ram-Lak';
    case 6
        error('Ambiguous input arguments')
    case 7
        FOVx = varargin{1};
        FOVy = varargin{2};
        tframe = [1:Nsets];
        tip = [30 50 60 60 100 20 20 192 255]'*ones(1,Nsets);
        SNR = [];
        interp_method = 'linear';
        filter = 'Ram-Lak';
    case 8
        FOVx = varargin{2};
        FOVy = varargin{3};
        tframe = [1:Nsets];
        tip = varargin{1};
        SNR = [];
        interp_method = 'linear';
        filter = 'Ram-Lak';
    case 9
        FOVx = varargin{3};
        FOVy = varargin{4};
        tframe = varargin{2};
        tip = varargin{1};
        SNR = [];
        interp_method = 'linear';
        filter = 'Ram-Lak';
    case 10
        FOVx = varargin{3};
        FOVy = varargin{4};
        tframe = varargin{2};
        tip = varargin{1};
        SNR = varargin{5};
        interp_method = 'linear';
        filter = 'Ram-Lak';
    otherwise
        FOVx = varargin{3};
        FOVy = varargin{4};
        tframe = varargin{2};
        tip = varargin{1};
        SNR = varargin{5};
        interp_method = varargin{6};
        filter = varargin{7};
end

switch length(Nsiz)
    case 1
        Nx = Nsiz;
        Ny = Nsiz;
     case 2
         Nx = Nsiz(1);
         Ny = Nsiz(2);
    otherwise
        error('Invalid dimension: Nsiz')
end
I = zeros(Ny,Nx,Nsets);

if isempty(tip)
    tframe = 0:120:120*(Nsets-1);
    CA = tcac(tframe);
    RI = c2i(CA);
    tip = [30 50 60 60 100 20 20 192 255]'*ones(1,Nsets);
    tip(1,:) = 240*RI(4,:);
    tip(3,:) = 180*RI(3,:);
    tip(4,:) = 180*RI(2,:);
    tip(6,:) = 220*RI(1,:);
    tip(7,:) = 220*RI(1,:);
end

rad = theta*pi/180;
txy = [FOVx*txy(1,:)/Nx; FOVy*txy(2,:)/Ny];

[fc1] = dcekr('f', tframe, tip(9,:), theta, txy, 'ellipse', 43.7, 1.0, 0.75, Nsiz, FOVx, FOVy, SNR);

R = [cos(rad(:)) -sin(rad(:)); sin(rad(:)) cos(rad(:))];
xy = R*[0;-1.5];
xy = reshape(xy,Nsets,2)';
[fc2] = dcekr('f', tframe, tip(9,:)-tip(8,:), theta, xy+txy, 'ellipse', 41.3, 1.0, 0.75, Nsiz, FOVx, FOVy, SNR);

xy = R*[0;23.4];
xy = reshape(xy,Nsets,2)';
[fc3] = dcekr('f', tframe, tip(1,:)-tip(8,:), theta, xy+txy, 'ellipse', 12.3, 0.9, 0.75, Nsiz, FOVx, FOVy, SNR);

xy = R*[0;7.5];
xy = reshape(xy,Nsets,2)';
[fc4] = dcekr('f', tframe, tip(2,:)-tip(8,:), theta, xy+txy, 'ellipse', 2.95, 1.0, 0.9, Nsiz, FOVx, FOVy, SNR);

xy = R*[-0.7;-9.5];
xy = reshape(xy,Nsets,2)';
[fc5] = dcekr('f', tframe, tip(2,:)-tip(8,:), theta, xy+txy, 'ellipse', 2.95, 1.0, 0.9, Nsiz, FOVx, FOVy, SNR);

xy = R*[15;1.3];
xy = reshape(xy,Nsets,2)';
[fc6] = dcekr('f', tframe, tip(3,:)-tip(8,:), theta-17, xy+txy, 'ellipse', 11.7, 1.65, 0.6, Nsiz, FOVx, FOVy, SNR);

xy = R*[-15;0.4];
xy = reshape(xy,Nsets,2)';
[fc7] = dcekr('f', tframe, tip(4,:)-tip(8,:), theta+17, xy+txy, 'ellipse', 20.3, 1.95, 0.7, Nsiz, FOVx, FOVy, SNR);

xy = R*[0;-39.3];
xy = reshape(xy,Nsets,2)';
[fc8] = dcekr('f', tframe, tip(5,:)-tip(8,:), theta, xy+txy, 'ellipse', 1.45, 1, 1, Nsiz, FOVx, FOVy, SNR);

xy = R*[4.2;-39.3];
xy = reshape(xy,Nsets,2)';
[fc9] = dcekr('f', tframe, tip(6,:)-tip(8,:), theta, xy+txy, 'ellipse', 2.75, 1.8, 0.6, Nsiz, FOVx, FOVy, SNR);

xy = R*[-6.5;-39.3];
xy = reshape(xy,Nsets,2)';
[fc10, kx, ky] = dcekr('f', tframe, tip(7,:)-tip(8,:), theta, xy+txy, 'ellipse', 2.45, 0.6, 1.8, Nsiz, FOVx, FOVy, SNR);

f = fc1-fc2+fc3+fc4+fc5+fc6+fc7+fc8+fc9+fc10;

% if ~isempty(SNR)
%     A(1) = pi*(43.7)*(43.7/0.75);
%     A(2) = pi*(41.3)*(41.3/0.75);
%     A(3) = pi*(12.3/0.9)*(12.3/0.75);
%     A(4) = pi*(2.95)*(2.95/0.9);
%     A(5) = pi*(2.95)*(2.95/0.9);
%     A(6) = pi*(11.7/1.65)*(11.7/0.6);
%     A(7) = pi*(20.3/1.95)*(20.3/0.7);
%     A(8) = pi*(1.45)*(1.45);
%     A(9) = pi*(2.75/1.8)*(2.75/0.6);
%     A(10) = pi*(2.45/0.6)*(2.45/1.8);
%     
%     A(1) = (A(1)-A(2));
%     A(2) = (A(2)-sum(A(3:end)));
%     
%     T = [tip(9,:);tip(8,:);tip(1:2,:);tip(2,:);tip(3:7,:)];
%     sigmakx = sqrt((Nx*Ny)*((sum(A*(T.^2))/Nsets)/sum(A))*power(10,-SNR/10)/(2));
%     N = random('Normal',0,sigmakx,prod(size(f)),1)+sqrt(-1)*random('Normal',0,sigmakx,prod(size(f)),1);
%     N = reshape(N,size(f));
%     f = f+N;
% end

switch upper(datatype)
    case 'F'
        if nargout == 2
            varargout{1} = f;
            varargout{2} = size(f);
        else
            varargout {1}= f;
        end
    otherwise
        for i = 1:Nsets
%             I(:,:,i) = abs(Recon_arbitrarykspace(kx,ky,squeeze(f(:,:,i)),Nx,Ny,'linear',FOVx,FOVy));
            Nmax = max(Nsiz);
            I(:,:,i) = abs(fbprec(squeeze(f(:,:,i)),[0:Nmax-1]*180/Nmax,Nmax,interp_method,filter));
        end
        if nargout == 2
            varargout{1} = I;
            varargout{2} = size(I);
        else
            varargout{1} = I;
        end
end